# http://en.wikipedia.org/wiki/Tanh-sinh_quadrature
# David H. Bailey, Karthik Jeyabalan, and Xiaoye S. Li, "23A comparison of three high-precision quadrature schemes". Experimental Mathematics, 14.3 (2005).

#from numpy import arange, pi, cosh, sinh, tanh, sum
from numpy import *

from quad_test import *

debug_output = 0
#debug_output = 1

def de_quad_rule(h, n):
    j = arange(-n, n+1)
    t = h*j
    u1 = pi/2.0 * cosh(t)
    u2 = pi/2.0 * sinh(t)
    x = tanh(u2)
    w = u1 / cosh(u2)**2
    return x, w * h

def de_quad_base(f, h, n):
    x, w = de_quad_rule(h, n)
    return sum(w * f(x))

def de_quad(g, a, b, h, n):
    d = (b - a) / 2.0
    def f(x):
        return g(a + (x + 1.0) * d)
    return de_quad_base(f, h, n) * d

def test_de_quad(g, h, n):
    a, b = g.low, g.high
    I = de_quad(f, a, b, h, n)
    if debug_output:
        print g.exact, I, I - g.exact
    return abs(I - g.exact)

def run1(f):
    n = 20
    h = 2.0/n
    print test_de_quad(f, h, n)

def run2(f):
    ns = 2**arange(9)
    qs = range(1, 5+1)
    table = [ [ test_de_quad(f, h, n) 
                for h in [ float(q) / n for q in qs ] ]
              for n in ns ]
    print "\n".join([ " ".join([ "%5.0e" % el for el in row ]) 
                      for row in table ])

if __name__ == "__main__":
    for f in [ f0, f1, f5, f6, semicirc, sinus, flatends ]:
        print f.__name__
        run2(f)
