"""spectral analysis of a two-grid iteration
for the heat equation on a uniform periodic grid
using red-black Gauss-Seidel, standard coarsening,
full weighting restriction and bilinear interpolation
"""

from mynum import *
from mypyx import *
#text.set(mode="latex")

from points import in_ex, ex_in, in_in

def maxabs(x):
    return maximum.reduce(absolute(x.flat))

def sigma(w_x, w_y, z=0):
    if w_x == 1 and w_y == 1:
        return zeros(4)
    mm = matrixmultiply
    sle = solve_linear_equations
    w_x_ = 1 / w_x
    w_y_ = 1 / w_y
    I4 = identity(4)
    I1 = identity(1)
    L = array([[-4, 1+w_y, 1+w_x, 0],
               [1+w_y_, -4, 0, 1+w_x],
               [1+w_x_, 0, -4, 1+w_y],
               [0, 1+w_x_, 1+w_y_, -4]])
    L = z * I4 - L
    R = array([[4, 2*(1+w_y), 2*(1+w_x), 1+w_x+w_y+w_x*w_y]]) / 16
    Lc = array([[-4+w_x+w_x_+w_y +w_y_]]) / 4
    Lc = z * I1 - Lc
    P = (array([[4],
                [2*(1+w_y_)],
                [2*(1+w_x_)],
                [1+w_x_+w_y_+w_x_*w_y_]]) / 4)
    Sr0 = array([[4+z, 0, 0, 0],
                 [0, 1, 0, 0],
                 [0, 0, 1, 0],
                 [0, 0, 0, 4+z]])
    Sr1 = array([[0, 1+w_y, 1+w_x, 0],
                 [0, 1, 0, 0],
                 [0, 0, 1, 0],
                 [0, 1+w_x_, 1+w_y_, 0]])
    Sb0 = array([[1, 0, 0, 0],
                 [0, 4+z, 0, 0],
                 [0, 0, 4+z, 0],
                 [0, 0, 0, 1]])
    Sb1 = array([[1, 0, 0, 0],
                 [1+w_y_, 0, 0, 1+w_x],
                 [1+w_x_, 0, 0, 1+w_y],
                 [0, 0, 0, 1]])
    S = mm(sle(Sb0, Sb1), sle(Sr0, Sr1))
    S1 = S
    #S2 = I4
    S2 = S
    M = mm(S2, mm(I4 - mm(P, sle(Lc, mm(R, L))), S1))
    ew = eigenvalues(M)
    return ew

n_x = 2**4
n_y = 2**4
n_t = 2**6
t_x = in_ex(n_x, 0, 2*pi)
t_y = in_ex(n_y, 0, 2*pi)
#t_t = in_ex(n_t, 0, 2*pi)
t_t = in_in(n_t+1, -pi, pi)
w_x = exp(1j * t_x)
w_y = exp(1j * t_y)
w_t = exp(1j * t_t)
#xi = in_ex(n_t, 0, 4)
#z = 1j * xi
#z = [ 3.0j ]
dt = 1.0
z = (1.0 - w_t) / dt

ew = zeros((n_x, n_y, n_t, 4), Complex)
rho = 0
rho_ind = []
rho_quot = 0
for i in range(n_x):
    for j in range(n_y):
        for k in range(n_t):
            ew_ = sigma(w_x[i], w_y[j], z[k])
            ew_abs = absolute(ew_)
            ind = argsort(ew_abs)
            ew[i,j,k] = take(ew_, ind)
            rho_ = ew_abs[ind[-1]]
            if rho_ > rho:
                rho = rho_
                rho_ind = [(i, j, k)]
            elif rho_ == rho:
                rho_ind.append([i, j, k])
            if ew_abs[ind[-2]] != 0.0:
            #if ew_abs[ind[-2]] >= 1e-2 and ew_abs[ind[-1]] >= 1e-2:
                rho_quot_ = ew_abs[ind[-2]] / ew_abs[ind[-1]]
                rho_quot = max(rho_quot, rho_quot_)
                
print rho_ind
print [ (w_x[i], w_y[j], z[k]) for (i,j,k) in rho_ind ]
print [ ew[i,j,k] for (i,j,k) in rho_ind ]
print [ maxabs(ew[i,j,k]) for (i,j,k) in rho_ind ]
print maxabs(ew)
print rho_quot

#assert maximum.reduce(absolute(ew.flat.imag)) <= 1e-12
#ew_quot = absolute((ew[...,3]))/(absolute(ew[...,2]+1e-17))
#print maximum.reduce(ew_quot.flat)
#print maximum.reduce(1.0 / ew_quot.flat)


# cumulative probability density
def cpd_graph(ew):
    #ew_ = ew.flat.real
    ew_ = absolute(ew.flat)
    n = len(ew_)
    g = graph.graphxy(width=10,
                      x=graph.axis.lin(title=r"$\lambda$"),
                      y=graph.axis.lin(title="cpd"))
    g.plot(datalist(sort(ew_), ex_in(n, 0, 1)),
           [graph.style.line()])
    return g

def rho_imag_graph(ew, xi):
    rho = maximum.reduce(maximum.reduce(maximum.reduce(absolute(ew), 3)))
    g = graph.graphxy(width=10,
                      x=graph.axis.lin(title=r"$\xi$",min=min(xi),max=max(xi)),
                      y=graph.axis.lin(title=r"$\rho$"))
    g.plot(datalist(xi, rho), [graph.style.line()])
    return g

def x_y_graph(ew):
    # assume there are only two non-zero eigenvalues for
    # each w_x - w_y pair and that they are real
#     print shape(ew)
#     print ew
#     ind = argsort(absolute(ew), -1)
#     print ind
#     ew = take(ew.real, ind, -1)
#     print ew
#     print shape(ew)
    #ew_ = 
    #print "_", ew_
    #print shape(ew_)
    #ew = reshape(transpose(ew.real, (2, 3, 0, 1)),
    ew = reshape(transpose(absolute(ew), (2, 3, 0, 1)),
                 (ew.shape[2], ew.shape[3], ew.shape[0]*ew.shape[1]))
    g = graph.graphxy(width=10, height=10,
                      x=graph.axis.lin(title=r"$\lambda_1$"),
                      y=graph.axis.lin(title=r"$\lambda_2$"))
    for k in range(ew.shape[0]):
        #g.plot(datalist(ew[k,2,:], ew[k,3,:]))
        g.plot(datalist(ew[k,3,:], ew[k,2,:]))
    return g

def re_im_graph(ew):
    g = graph.graphxy(width=10,
                      x=graph.axis.lin(title=r"$re$"),
                      y=graph.axis.lin(title=r"$im$"))
    g.plot(datalist(ew.flat.real, ew.flat.imag),
           [graph.style.symbol()])
    return g

#g = x_y_graph(ew)
g = re_im_graph(ew)
g.writetofile("mg_2d_re_im.eps")
g = rho_imag_graph(ew, t_t)
g.writetofile("mg_2d_rho_imag.eps")
